package Model_Viewer is

   Max_Dependents : constant := 10;

   type Model_Type is abstract tagged private;
   type Model_Ptr is access all Model_Type'Class;

   type View_Type is abstract tagged private;
   type View_Ptr is access all View_Type'Class;

   -- Operations for Models

   procedure Add_Dependent (Model : in out Model_Type;
                            View  : in     View_Ptr);

   function Value (Model : in Model_Type) return Integer is abstract;

   -- Operations for Views

   procedure Update (View : in out View_Type) is abstract;

private

   procedure Change (Model : in out Model_Type);

   subtype Count_Range is Integer     range 0..Max_Dependents;
   subtype Index_Range is Count_Range range 1..Max_Dependents;

   type Viewer_Array is array (Index_Range) of View_Ptr;

   type Model_Type is abstract tagged
      record
         Dependents     : Viewer_Array;
         Num_Dependents : Count_Range    := 0;
      end record;

   type View_Type is abstract tagged
      record
         My_Model : Model_Ptr;
      end record;

end Model_Viewer;
